// START A3HEADER
//
// The Atlantis program, and all of it's source code (including
// this file) is Copyright (c) 1995-1998 by Geoff Dunbar. This
// source code is distributed under the terms of the license
// agreement in the file LICENSE.txt. If this file was not included
// with the source code you received, you may not use, modify, or
// distribute this source code.
//
// See the Atlantis Project web page for details:
// http://www.prankster.com/project
//
// END A3HEADER
#ifndef FACTION_CLASS
#define FACTION_CLASS

class Faction;
class Game;

#include "gameio.h"
#include "aregion.h"
#include "fileio.h"
#include "unit.h"
#include "battle.h"
#include "skills.h"
#include "alist.h"
#include "astring.h"

enum {
    A_HOSTILE,
    A_UNFRIENDLY,
    A_NEUTRAL,
    A_FRIENDLY,
    A_ALLY,
    NATTITUDES
};

enum {
    F_WAR,
    F_TRADE,
    F_MAGIC,
    NFACTYPES
};

enum {
    TEMPLATE_OFF,
    TEMPLATE_SHORT,
    TEMPLATE_LONG
};

enum {
    QUIT_NONE,
    QUIT_BY_ORDER,
    QUIT_BY_GM,
    QUIT_AND_RESTART,
    QUIT_WON_GAME,
    QUIT_GAME_OVER,
};

extern char ** AttitudeStrs;
extern char ** FactionStrs;

int ParseAttitude(AString *);

int MagesByFacType(int);

class FactionVector {
public:
  FactionVector(int);
  ~FactionVector();

  void ClearVector();
  void SetFaction(int, Faction *);
  Faction *GetFaction(int);

  Faction **vector;
  int vectorsize;
};
  
class Attitude : public AListElem {
public:
  Attitude();
  ~Attitude();
  void Writeout(Aoutfile * );
  void Readin( Ainfile *, ATL_VER version );
  
  int factionnum;
  int attitude;
};

class FactionPtr : public AListElem {
public:
  Faction * ptr;
};

class Faction : public AListElem
{
public:
    Faction();
    Faction(int);
    ~Faction();
    
    void Readin( Ainfile *, ATL_VER version );
    void Writeout( Aoutfile * );
    void View();
    
    void SetName(AString *);
    void SetNameNoChange( AString *str );
    void SetAddress( AString &strNewAddress );
    
    void CheckExist(ARegionList *);
    void Error(const AString &);
    void Event(const AString &);
    
    AString FactionTypeStr();
    void WriteReport( Areport *f, Game *pGame );
    void WriteFacInfo(Aoutfile *);
    
    void SetAttitude(int,int); /* faction num, attitude */
    /* if attitude == -1, clear it */
    int GetAttitude(int);
    void RemoveAttitude(int);
    
    int CanCatch(ARegion *,Unit *);
    int CanSee(ARegion *,Unit *); /* Return 1 if can see, 2 if can see
				   faction */
    
    void DefaultOrders();
    void TimesReward();
    
    void SetNPC();
    int IsNPC();
    
    int num;

    //
    // The type is only used if Globals->FACTION_LIMIT_TYPE ==
    // FACLIM_FACTION_TYPES
    //
    int type[NFACTYPES];

    int lastchange;
    int lastorders;
    int unclaimed;
    AString * name;
    AString * address;
    AString * password;
    int times;
    int temformat;
    char exists;
    int quit;
    int numshows;
    
    int nummages;
    AList war_regions;
    AList trade_regions;

    /* Used when writing reports */
    AList present_regions;
    
    int defaultattitude;
    AList attitudes;
    SkillList skills;
	
    //
    // Both are lists of AStrings
    //
    AList extraPlayers;
    AList errors;
    AList events;
    AList battles;
    AList shows;
    AList itemshows;
};

Faction * GetFaction(AList *,int);
Faction * GetFaction2(AList *,int); /*This AList is a list of FactionPtr*/

#endif
